# -*- coding: utf-8 -*-

# MIT License
# 
# Copyright (c) 2017 Sean-Jiun Wang
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
cms_figure
==========

This module provides classes and functions for styling plots made
using ROOT according to the CMS Publication Committee standards.

:copyright: (c) 2017 by Sean-Jiun Wang.
:license: MIT, see LICENSE for more details.
"""

__version__ = '0.9.0'

# Core classes
from .labels import CMSLabel, LuminosityLabel

from .tdr_style import TDRStyle

# Utilities
from .utils import draw_labels

__all__ = [
    # Core classes
    'CMSLabel', 'LuminosityLabel', 'TDRStyle',

    # Utilities
    'draw_labels',
]




# MIT License
# 
# Copyright (c) 2017 Sean-Jiun Wang
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import ROOT

from .labels import CMSLabel, LuminosityLabel


def draw_labels(lumi_text, cms_position='left', extra_text=''):
    """Draw the CMS Publication Committee figure labels on the active canvas.

    Parameters
    ----------
    lumi_text : string
        The luminosity label text. Data taking periods must be separated by
        the "+" symbol, e.g. "19.7 fb^{-1} (8 TeV) + 4.9 fb^{-1} (7 TeV)".
    cms_position : string, optional
        The CMS label position on the active canvas:
            :left: The top left corner inside the frame (default)
            :center: The top center inside the frame
            :right: The top right corner inside the frame
            :outside: The top left corner outside the frame
    extra_text : string, optional
        The sublabel text positioned below the CMS label inside of the frame
        or to the right of the CMS label outside of the frame. Common examples
        are "Preliminary", "Simulation", or "Unpublished". The default is an
        empty string for no sublabel.
    """
    cms_label = CMSLabel()
    cms_label.position = cms_position
    cms_label.sublabel.text = extra_text
    cms_label.draw()
    lumi_label = LuminosityLabel(lumi_text)
    lumi_label.draw()
    ROOT.gPad.Update()




import ast
import re

from setuptools import setup, find_packages


try:
    import ROOT
except ImportError:
    raise ImportError('Unable to import ROOT. Please check that ROOT and its Python bindings are installed.')
ROOT.PyConfig.IgnoreCommandLineOptions = True


VERSION_RE = re.compile(r'__version__\s+=\s+(.*)')


with open('cms_figure/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(VERSION_RE.search(f.read().decode('utf-8')).group(1)))


setup(
    name='cms_figure',
    version=version,
    packages=find_packages(),
    description='Styling and labels for CMS figures produced using ROOT',
    author='Sean-Jiun Wang',
    author_email='sean.jiun.wang@gmail.com',
    maintainer='Sean-Jiun Wang',
    maintainer_email='sean.jiun.wang@gmail.com',
    url='https://github.com/swang373/cms_figure',
    download_url='https://github.com/swang373/cms_figure/tarball/{0}'.format(version),
    license='MIT',
    python_requires='>=2.7, <3',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Physics',
        'Topic :: Scientific/Engineering :: Visualization',
        'Topic :: Software Development',
        'Topic :: Utilities',
    ],
)




# MIT License
# 
# Copyright (c) 2017 Sean-Jiun Wang
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import ROOT


class TDRStyle(ROOT.TStyle):
    """The CMS Technical Design Report (TDR) plotting style.

    Unused styling options in the original definition have been removed
    for clarity.
    """
    def __init__(self):
        super(TDRStyle, self).__init__('tdrStyle', 'Style for P-TDR')
        # Canvas
        self.SetCanvasBorderMode(0)
        self.SetCanvasColor(0)
        self.SetCanvasDefH(600) # Height
        self.SetCanvasDefW(600) # Width
        self.SetCanvasDefX(0) # On-Screen Position
        self.SetCanvasDefY(0)
        # Pad
        self.SetPadBorderMode(0)
        self.SetPadColor(0)
        self.SetPadGridX(False)
        self.SetPadGridY(False)
        self.SetGridColor(0)
        self.SetGridStyle(3)
        self.SetGridWidth(1)
        # Frame
        self.SetFrameBorderMode(0)
        self.SetFrameBorderSize(1)
        self.SetFrameFillColor(0)
        self.SetFrameFillStyle(0)
        self.SetFrameLineColor(1)
        self.SetFrameLineStyle(1)
        self.SetFrameLineWidth(1)
        # Histogram
        self.SetHistLineColor(1)
        self.SetHistLineStyle(0)
        self.SetHistLineWidth(1)
        self.SetEndErrorSize(2)
        self.SetMarkerStyle(20)
        # Fit/Function
        self.SetOptFit(1)
        self.SetFitFormat('5.4g')
        self.SetFuncColor(2)
        self.SetFuncStyle(1)
        self.SetFuncWidth(1)
        # Date
        self.SetOptDate(0)
        # Statistics Box
        self.SetOptFile(0)
        self.SetOptStat(0) # Pass 'mr' to display the mean and RMS.
        self.SetStatColor(0)
        self.SetStatFont(42)
        self.SetStatFontSize(0.025)
        self.SetStatTextColor(1)
        self.SetStatFormat('6.4g')
        self.SetStatBorderSize(1)
        self.SetStatH(0.1)
        self.SetStatW(0.15)
        # Margins
        self.SetPadTopMargin(0.05)
        self.SetPadBottomMargin(0.13)
        self.SetPadLeftMargin(0.16)
        self.SetPadRightMargin(0.02)
        # Global Title
        self.SetOptTitle(0) # 0 = No Title
        self.SetTitleFont(42)
        self.SetTitleColor(1)
        self.SetTitleTextColor(1)
        self.SetTitleFillColor(10)
        self.SetTitleFontSize(0.05)
        # Axis Titles
        self.SetTitleColor(1, 'XYZ')
        self.SetTitleFont(42, 'XYZ')
        self.SetTitleSize(0.06, 'XYZ')
        self.SetTitleXOffset(0.9)
        self.SetTitleYOffset(1.25)
        # Axis Labels
        self.SetLabelColor(1, 'XYZ')
        self.SetLabelFont(42, 'XYZ')
        self.SetLabelOffset(0.007, 'XYZ')
        self.SetLabelSize(0.05, 'XYZ')
        # Axes
        self.SetAxisColor(1, 'XYZ')
        self.SetStripDecimals(True)
        self.SetTickLength(0.03, 'XYZ')
        self.SetNdivisions(510, 'XYZ')
        self.SetPadTickX(1) # 0 = Text labels (and ticks) only on bottom, 1 = Text labels on top and bottom
        self.SetPadTickY(1)
        # Log Scale Axes
        self.SetOptLogx(0)
        self.SetOptLogy(0)
        self.SetOptLogz(0)
        # Postscript Options
        self.SetPaperSize(20, 20)
        # Hatches
        self.SetHatchesLineWidth(5)
        self.SetHatchesSpacing(0.05)

    def __enter__(self):
        """Set the gStyle to tdrStyle while remembering the previous gStyle."""
        self.previous_gStyle = ROOT.gROOT.GetStyle(ROOT.gStyle.GetName())
        self.cd()
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        """Reset to the previous gStyle.
        """
        self.previous_gStyle.cd()




# MIT License
# 
# Copyright (c) 2017 Sean-Jiun Wang
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import ROOT


class LabelTextAlignmentError(Exception):
    pass


class LabelPositionError(Exception):
    pass


class LabelBase(ROOT.TLatex):
    """The base label class."""

    # Tuples of horizontal and vertical text alignment names and
    # their corresponding ROOT text alignment integer values.
    TEXT_ALIGNMENT = {
        ('left', 'bottom'): 11,
        ('left', 'center'): 12,
        ('left', 'top'): 13,
        ('center', 'bottom'): 21,
        ('center', 'center'): 22,
        ('center', 'top'): 23,
        ('right', 'bottom'): 31,
        ('right', 'center'): 32,
        ('right', 'top'): 33,
    }

    def __init__(self):
        super(LabelBase, self).__init__()

    @property
    def align(self):
        return self.GetTextAlign()

    @align.setter
    def align(self, value):
        if value in self.TEXT_ALIGNMENT:
            self.SetTextAlign(self.TEXT_ALIGNMENT[value])
        elif value in self.TEXT_ALIGNMENT.values():
            self.SetTextAlign(value)
        else:
            raise LabelTextAlignmentError('Unrecognized value: {0!s}'.format(value))

    @property
    def font(self):
        return self.GetTextFont()

    @font.setter
    def font(self, value):
        self.SetTextFont(value)

    @property
    def size(self):
        return self.GetTextSize()

    @size.setter
    def size(self, value):
        self.SetTextSize(value)

    @staticmethod
    def get_canvas_margins():
        """Return the top, right, bottom, and left margins of the active canvas.

        Figure labels are often oriented relative to these margins.
        """
        return ROOT.gPad.GetTopMargin(), ROOT.gPad.GetRightMargin(), ROOT.gPad.GetBottomMargin(), ROOT.gPad.GetLeftMargin()


class CMSLabel(LabelBase):
    """A label displaying the CMS name.

    Credits to Gautier Hamel de Monchenault (Saclay), Joshua Hardenbrook (Princeton),
    and Dinko Ferencek (Rutgers) for the initial Python implementation.

    The label's text and style properties, which are set to default values from the
    CMS Publication Committee, are exposed as instance attributes for customization:

    text : string
        The CMS label text. The default is "CMS".
    position : string
        One of the following label positions on the active canvas:
            :left: The top left corner inside the frame (default)
            :center: The top center inside the frame
            :right: The top right corner inside the frame
            :outside: The top left corner outside the frame
    font : int
        The text font code. The default is 61 (Helvetica Bold).
    scale : float
        The text size scale relative to the size of the top margin
        of the active canvas. The default is 0.75.
    padding_left : float
        The amount of padding to the left of the text when positioned
        inside the frame, relative to the frame width of the active canvas.
        The default is 0.045.
    padding_right : float
        The amount of padding to the right of the text when positioned
        inside the frame, relative to the frame width of the active canvas.
        The default is 0.045.
    padding_top : float
        The amount of padding above the text. When positioned inside of
        the frame, it is relative to the frame height of the active canvas
        and the default is 0.035. When positioned outside of the frame, it
        is relative to the size of the top margin of the active canvas and
        the default is 0.8.
    sublabel.text : string
        The sublabel text positioned below the main text inside of the frame
        or to the right of the main text outside of the frame. Common examples
        are "Preliminary", "Simulation", or "Unpublished". The default is an
        empty string for no sublabel.
    sublabel.font : string
        The sublabel text font code. The default is 52 (Helvetica Italic).
    sublabel.scale : float
        The sublabel text size scale relative to the size of the main text.
        The default is 0.76.
    sublabel.padding_left : float
        The amount of padding to the left of the sublabel text relative to the
        frame width of the active canvas. This only applies if the main text is
        positioned outside the frame of the active canvas. The default is 0.12.
    sublabel.padding_top : float
        The amount of padding above the sublabel text relative to the size of
        the main text. This only applies if the main text is positioned inside
        the frame of the active canvas. The default is 1.2.
    """
    def __init__(self):
        super(CMSLabel, self).__init__()
        self.text = 'CMS'
        self.position = 'left'
        self.font = 61
        self.scale = 0.75
        self.padding_left = 0.045
        self.padding_right = 0.045
        self.padding_top = None
        self.sublabel = LabelBase()
        self.sublabel.text = ''
        self.sublabel.font = 52
        self.sublabel.scale = 0.76
        self.sublabel.padding_left = 0.12
        self.sublabel.padding_top = 1.2

    def draw(self):
        """Draw the CMS label and sublabel on the active canvas."""
        # Draw the label.
        if self.position == 'left':
            label_coordinates = self._draw_label_left()
        elif self.position == 'center':
            label_coordinates = self._draw_label_center()
        elif self.position == 'right':
            label_coordinates = self._draw_label_right()
        elif self.position == 'outside':
            label_coordinates = self._draw_label_outside()
        else:
            raise LabelPositionError('Unrecognized value: {0}'.format(self.position))
        # Draw the sublabel.
        if self.sublabel.text:
            if self.position == 'outside':
                self._draw_sublabel_outside(*label_coordinates)
            else:
                self._draw_sublabel_inside(*label_coordinates)

    def _draw_label_left(self):
        """Draw the label on the top left corner inside the frame and return its coordinates."""
        top_margin, right_margin, bottom_margin, left_margin = self.get_canvas_margins()
        self.size = self.scale * top_margin
        self.align = ('left', 'top')
        x = left_margin + self.padding_left * (1 - left_margin - right_margin)
        y = 1 - top_margin - (self.padding_top or 0.035) * (1 - top_margin - bottom_margin)
        self.DrawLatexNDC(x, y, self.text)
        return x, y

    def _draw_label_center(self):
        """Draw the label on the top center inside the frame and return its coordinates."""
        top_margin, right_margin, bottom_margin, left_margin = self.get_canvas_margins()
        self.size = self.scale * top_margin
        self.align = ('center', 'top')
        x = left_margin + 0.5 * (1 - left_margin - right_margin)
        y = 1 - top_margin - (self.padding_top or 0.035) * (1 - top_margin - bottom_margin)
        self.DrawLatexNDC(x, y, self.text)
        return x, y

    def _draw_label_right(self):
        """Draw the label on the top right corner inside the frame and return its coordinates."""
        top_margin, right_margin, bottom_margin, left_margin = self.get_canvas_margins()
        self.size = self.scale * top_margin
        self.align = ('right', 'top')
        x = 1 - right_margin - self.padding_right * (1 - left_margin - right_margin)
        y = 1 - top_margin - (self.padding_top or 0.035) * (1 - top_margin - bottom_margin)
        self.DrawLatexNDC(x, y, self.text)
        return x, y

    def _draw_label_outside(self):
        """Draw the label on the top left corner outside the frame and return its coordinates."""
        top_margin, _, _, left_margin = self.get_canvas_margins()
        self.size = self.scale * top_margin
        self.align = ('left', 'bottom')
        x = left_margin
        y = 1 - (self.padding_top or 0.8) * top_margin
        self.DrawLatexNDC(x, y, self.text)
        return x, y

    def _draw_sublabel_inside(self, x_label, y_label):
        """Draw the sublabel below the label inside the frame."""
        self.sublabel.size = self.sublabel.scale * self.size
        self.sublabel.align = self.align
        x_sublabel = x_label
        y_sublabel = y_label - self.sublabel.padding_top * self.size
        self.sublabel.DrawLatexNDC(x_sublabel, y_sublabel, self.sublabel.text)

    def _draw_sublabel_outside(self, x_label, y_label):
        """Draw the sublabel to the right of the label outside the frame."""
        _, right_margin, _, left_margin = self.get_canvas_margins()
        self.sublabel.size = self.sublabel.scale * self.size
        self.sublabel.align = self.align
        x_sublabel = left_margin + self.sublabel.padding_left * (1 - left_margin - right_margin)
        y_sublabel = y_label
        self.sublabel.DrawLatexNDC(x_sublabel, y_sublabel, self.sublabel.text)


class LuminosityLabel(LabelBase):
    """A label displaying the integrated luminosity and center-of-mass energy.

    Credits to Gautier Hamel de Monchenault (Saclay), Joshua Hardenbrook (Princeton),
    and Dinko Ferencek (Rutgers) for the initial Python implementation.

    The label's text and style properties, which are set to default values from the
    CMS Publication Committee, are exposed as instance attributes for customization:

    font : int
        The text font code. The default is 42 (Helvetica).
    scale : float
        The text size scale relative to the size of the top margin
        of the active canvas. The default is 0.6.
    align : int or 2-tuple of strings
        The text alignment relative to the drawing coordinates of the text as
        either an integer code or tuple of horizontal and vertical alignment
        names, respectively. The default is 31, or ("right", "bottom").
    padding_top : float
        The amount of padding above the text relative to the size
        of the top margin of the active canvas. The default is 0.8.

    Parameters
    ----------
    text : string
        The luminosity label text. Data taking periods must be separated by
        the "+" symbol, e.g. "19.7 fb^{-1} (8 TeV) + 4.9 fb^{-1} (7 TeV)".
    """
    def __init__(self, text):
        super(LuminosityLabel, self).__init__()
        self.text = text
        self.font = 42
        self.scale = 0.6
        self.align = 31
        self.padding_top = 0.8

    def draw(self):
        """Draw the luminosity label on the active canvas."""
        top_margin, right_margin, _, _ = self.get_canvas_margins()
        self.size = self.scale * top_margin
        x = 1 - right_margin
        y = 1 - self.padding_top * top_margin
        self.DrawLatexNDC(x, y, self.text)

